var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "typedoc/dist/lib/converter/components", "typedoc/dist/lib/output/components", "typedoc/dist/lib/output/events", "typedoc/dist/lib/models"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const components_1 = require("typedoc/dist/lib/converter/components");
    const components_2 = require("typedoc/dist/lib/output/components");
    const events_1 = require("typedoc/dist/lib/output/events");
    const models_1 = require("typedoc/dist/lib/models");
    /**
     * This plugin customizes some typedoc stuff for ui-router docs
     */
    let RenameNavLabelPlugin = class RenameNavLabelPlugin extends components_2.RendererComponent {
        /**
         * Rename the Default Theme's navigation labels:
         * - 'Globals' to 'Subsystems'
         * - 'Internals' to 'Public API'
         * - 'Externals' to 'Internal UI-Router API'
         */
        initialize() {
            this.listenTo(this.owner, {
                [events_1.RendererEvent.BEGIN]: this.onBeginRenderer,
            });
        }
        onBeginRenderer(event) {
            var options = this.application.options;
            this.labels = {
                globals: options.getValue('navigation-label-globals') || 'Subsystems',
                internals: options.getValue('navigation-label-internals') || 'Public API',
                externals: options.getValue('navigation-label-externals') || 'Internal UI-Router API',
            };
            let navigation = this.getNavigation();
            if (navigation)
                this.renameNavigationItems(navigation);
            this.useIndexForGlobals(event.project);
        }
        useIndexForGlobals(project) {
            function findGlobalReflection(ref) {
                if (!ref)
                    return undefined;
                if (ref.kind === models_1.ReflectionKind.Global)
                    return ref;
                return findGlobalReflection(ref.parent);
            }
            let externalModules = project.getReflectionsByKind(models_1.ReflectionKind.ExternalModule);
            externalModules
                .map(findGlobalReflection)
                .filter(x => !!x)
                .forEach(global => (global.url = 'index.html'));
        }
        renameNavigationItems(navigation) {
            navigation.children.forEach(item => {
                // Disable globals.html in favor of index.html
                if (item.isGlobals)
                    item.url = 'index.html';
                // Override labels of navigation items
                if (item.isGlobals && item.title === 'Globals')
                    item.title = this.labels.globals;
                if (item.isLabel && item.title === 'Internals')
                    item.title = this.labels.internals;
                if (item.isLabel && item.title === 'Externals')
                    item.title = this.labels.externals;
            });
        }
        getNavigation() {
            var components = this.application.renderer.getComponents();
            var navigationPlugin = components.filter(c => c.componentName === 'navigation')[0];
            if (!navigationPlugin) {
                return console.log('typedoc-plugin-ui-router: WARNING: NavigationPlugin not loaded');
            }
            if (!navigationPlugin['navigation']) {
                return console.log('typedoc-plugin-ui-router: WARNING: NavigationPlugin.navigation not loaded');
            }
            return navigationPlugin['navigation'];
        }
    };
    RenameNavLabelPlugin = __decorate([
        components_1.Component({ name: 'RenameNavLabel' })
    ], RenameNavLabelPlugin);
    exports.RenameNavLabelPlugin = RenameNavLabelPlugin;
});
